local L = AceLibrary("AceLocale-2.2"):new("ProfessionsBook")

ProfessionsBook = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0")

ProfessionsBook:RegisterDB("ProfessionsBookDB")

ProfessionsBook:RegisterDefaults('realm', {
	selectedChar = "",
	chars = {
		['*'] = { 
			minimap = 0,
			degr = 180,
			tradeSkills = {
				['*'] = { 
					level = 0,
					totalCount = 0,
					tradeItems = {
						['*'] = { 
							tradeItemName = "",
							tradeItemType = "",
							tradeItemLink = "",
							tradeItemRecipeLink = "",
							tools = "",
							reagents = {
								['*'] = { 
									count = 0,
									reagentItemName = "",
									reagentItemLink = ""
								}
							}
						}
					}
				}
			}
		}
	}
})



-- Create skill header table. Only expanded headers are added to this table, so the contents of this tables reflects all expanded headers. 
local expandedHeaders = {}
local allHeaders = {}
local placesList = {}
local tradeItemList = {}

local channels = {"SAY", "PARTY", "GUILD", "OFFICER", "RAID"}

local professions = {L["Alchemy"], L["Blacksmithing"], L["Enchanting"], L["Engineering"], L["Jewelcrafting"], L["Leatherworking"], L["Tailoring"], L["Cooking"], L["First Aid"] }

-- Create chat commands
local options = { 
    type='group',
    args = {
        show = {
			name = "show",
		    type = 'execute',
			desc = L["Opens ProfessionsBook"],
			func = "ShowFrame"
        },
		debug = {
			name = "debug",
			type = 'execute',
			desc = "Debuggin",
			func = "Debug"
		},
    },
}

-- Seach result element
local searchElement = {
	charName = "",
	tradeSkill = "",
	id = 0,
	tradeItemName = ""
}

local searchResult = {}

ProfessionsBook:RegisterChatCommand("/professionsbook", "/pb", options)

local showValue = false
local selectedTradeSkill = "none"
local firstTradeItem = 1
local selectedTradeItem = 0
local tradeItemCount = 0
local debug = false
local selectedChannel = 0
local rescan = false
local searchFrameIsShown = false
local selectedSearchItem = "none"
local rescanCount = 0

PROFESSIONSBOOK_TITLE = L["Professions Book"]
PROFESSIONSBOOK_SEND = L["Send"]
PROFESSIONSBOOK_OPTIONS = L["Options"]
PROFESSIONSBOOK_SHOWMINIMAPICON = L["Show minimap icon"]

-- Max count of trade skill selection buttons
PROFESSIONSBOOK_TRADESKILL_BUTTON_COUNT = 6

-- Max count of trade items visible
PROFESSIONSBOOK_TRADEITEM_BUTTON_COUNT = 8

PROFESSIONSBOOK_TRADEITEM_HEIGHT = 16

PROFESSIONSBOOK_SEARCHITEM_BUTTON_COUNT = 19

PROFESSIONSBOOK_SEARCHITEM_HEIGHT = 16

PROFESSIONSBOOK_AUCTION_DISPLAY_COUNT = 8

TradeSkillTypeColor = { };
TradeSkillTypeColor["optimal"]	= { r = 1.00, g = 0.50, b = 0.25 };
TradeSkillTypeColor["medium"]	= { r = 1.00, g = 1.00, b = 0.00 };
TradeSkillTypeColor["easy"]		= { r = 0.25, g = 0.75, b = 0.25 };
TradeSkillTypeColor["trivial"]	= { r = 0.50, g = 0.50, b = 0.50 };
TradeSkillTypeColor["header"]	= { r = 1.00, g = 0.82, b = 0 };

local AuctionScroll = function (...)
	ProfessionsBook:SetAuctionColors()
end

local ShowTooltip = function (...)
	ProfessionsBook:ShowTooltip()
end

function ProfessionsBook:Debug()
	if debug == false then
		self:Print("Debugging turned on.")
		debug = true
	else
		self:Print("Debugging turned off.")
		debug = false
	end
end

function ProfessionsBook:DebugPrint(string)
	if debug == true then
		self:Print(string)
	end
end

function ProfessionsBook:ShowFrame()
	local openProfessionsBook = false
	local msg, i

	-- Show professionsbook frame only, if at least one professions of one character has been scanned (there are entries in saved variables). 
	for k,v in pairs(ProfessionsBook.db.realm.chars) do 
		openProfessionsBook = true
	end
	
	 if openProfessionsBook == true then
		if ProfessionsBookFrame:IsVisible() then
			ProfessionsBookFrame:Hide()
		else
			UIDropDownMenu_Initialize(ProfessionsBookCharacterDropDownMenu, ProfessionsBookCharacterDropDownMenu_Initialize)	
			i = 0
			if self.db.realm.selectedChar ~= "" then
				while true do
					UIDropDownMenu_SetSelectedValue(ProfessionsBookCharacterDropDownMenu, i)
					if self.db.realm.selectedChar == UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu) then
						break
					end
					if UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu) == nil then
						UIDropDownMenu_SetSelectedValue(ProfessionsBookCharacterDropDownMenu, 0)
					end
					i = i + 1
				end
			else
				UIDropDownMenu_SetSelectedValue(ProfessionsBookCharacterDropDownMenu, 0)
			end
			UIDropDownMenu_Initialize(ProfessionsBookChannelsDropDownMenu, ProfessionsBookChannelsDropDownMenu_Initialize)
			UIDropDownMenu_SetSelectedValue(ProfessionsBookChannelsDropDownMenu, 0)	
			ProfessionsBookFrame:Show()
			ProfessionsBookSendButton:Disable()
			ProfessionsBook:UpdateTradeSkillSelection()
			ProfessionsBookDetailScrollFrame:Hide()
			ProfessionsBook:HideSearchFrame()
			searchFrameIsShown = false
		end
	end
end

function ProfessionsBook:HideSearchFrame()
	local button, i

	ProfessionsBookSearchTop:Hide()
	ProfessionsBookSearchTopRight:Hide()
	ProfessionsBookSearchBottom:Hide()
	ProfessionsBookSearchBottomRight:Hide()
	SearchResultScrollFrame:Hide()
	ProfessionsBookOpenSearchButton:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up")
	ProfessionsBookSearchBox:Hide()
	ProfessionsBookSearchButton:Hide()
	SearchResultHighlightFrame:Hide()
	searchFrameIsShown = false
	for i=1,PROFESSIONSBOOK_SEARCHITEM_BUTTON_COUNT do
		button = getglobal("ProfessionsBookSearchResultButton" .. i)
		button:Hide()
	end	
end

function ProfessionsBook:ShowSearchFrame()
	ProfessionsBookSearchTop:Show()
	ProfessionsBookSearchTopRight:Show()
	ProfessionsBookSearchBottom:Show()
	ProfessionsBookSearchBottomRight:Show()
	SearchResultScrollFrame:Show()
	ProfessionsBookOpenSearchButton:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up")
	ProfessionsBookSearchBox:Show()
	ProfessionsBookSearchButton:Show()
	searchFrameIsShown = true
end

function ProfessionsBook:OnInitialize()
	local playerName, x, y

	-- Set up minimap button
	playerName= UnitName("player")
	if self.db.realm.chars[playerName].minimap == 1 then 
		ProfessionsBookShowMinimapCheckButton:SetChecked(1)
		MinimapButton:Show()
	else
		ProfessionsBookShowMinimapCheckButton:SetChecked(nil)
		MinimapButton:Hide()
	end
	x, y = ProfessionsBook:CalculateMinimapCoords(self.db.realm.chars[playerName].degr)
	ProfessionsBookMinimapSlider:SetValue(ProfessionsBook.db.realm.chars[playerName].degr)
	MinimapButton:SetPoint("CENTER", "MinimapBackdrop", "CENTER", x, y)	
end

function ProfessionsBook:OnEnable()
	self:RegisterEvent("TRADE_SKILL_SHOW")
	self:RegisterEvent("TRADE_SKILL_UPDATE")
	self:RegisterEvent("CRAFT_SHOW")
	self:RegisterEvent("CRAFT_UPDATE")
	hooksecurefunc("AuctionFrameBrowse_Update", AuctionScroll)
	hooksecurefunc(GameTooltip, "SetAuctionItem", ShowTooltip)
	hooksecurefunc(GameTooltip, "SetBagItem", ShowTooltip)
	hooksecurefunc(GameTooltip, "SetLootItem", ShowTooltip)
	hooksecurefunc(GameTooltip, "SetHyperlink", ShowTooltip)
	hooksecurefunc(GameTooltip, "SetLootRollItem", ShowTooltip)
	hooksecurefunc(GameTooltip, "SetMerchantItem", ShowTooltip)
end

function ProfessionsBook:OnDisable()
    -- Called when the addon is disabled
end

function ProfessionsBook:Hide()
	local i, button

	for i = 1, PROFESSIONSBOOK_TRADEITEM_BUTTON_COUNT do
		button = getglobal("ProfessionsBookTradeItemButton" .. i)
		button:Hide()
	end
	TradeItemHighlightFrame:Hide()
	for i = 1, PROFESSIONSBOOK_TRADESKILL_BUTTON_COUNT do
		button = getglobal("ProfessionsBookTradeSkillButton" .. i)
		button:UnlockHighlight()
	end
	ProfessionsBookDetailScrollFrame:Hide()
	ProfessionsBookSendButton:Disable()
	SkillLevel:Hide()
end

function ProfessionsBook:CRAFT_SHOW()
	rescanCount = 0
	if GetCraftDisplaySkillLine() ~= nil then
		self:ScanSkills("Enchanting")
	end
end

function ProfessionsBook:CRAFT_UPDATE()
	if rescanCount <= 5 then
		if GetCraftDisplaySkillLine() ~= nil then
			self:ScanSkills("Enchanting")
		end
	else
		self:DebugPrint(L["Rescan error"])
	end
end

function ProfessionsBook:TRADE_SKILL_SHOW()
	rescanCount = 0
	self:ScanSkills("TradeSkill")
end

function ProfessionsBook:TRADE_SKILL_UPDATE()
	if rescanCount <= 5 then
		self:ScanSkills("TradeSkill")
	else
		self:DebugPrint(L["Rescan error"])
	end
end

function ProfessionsBook:ShowTooltip()
	local k,v, k2, v2, skillLevel, line, text, newText, a
	local mayLearnRecipe = {}
	local knowingRecipe = {}
	
	for k,v in pairs(professions) do
		skillLevel = self:GetSkillLevel("GameTooltip", v)
		if  skillLevel > -1 then
			line = getglobal("GameTooltip" .. "TextLeft" .. 1)
			text = line:GetText()
			a, _ = string.find(text, ': ') 
			if (a ~= nil) then
				text =  string.sub(text, a + 2)
			else
				return 0
			end
			knowingRecipe = self:GetCharsKnowingRecipe(text, v)	
			mayLearnRecipe = self:GetCharsMayLearnRecipe(v, skillLevel)
			mayLearnRecipe = self:RemoveDuplicates(knowingRecipe, mayLearnRecipe)
			if getn(knowingRecipe) + getn(mayLearnRecipe) > 0 then
				GameTooltip:AddLine(" ")
			end
			if getn(knowingRecipe)  > 0 then
				newText = L["Known"] .. ": "
				for k2, v2 in pairs(knowingRecipe) do
					newText = newText .. " " .. v2
				end
				GameTooltip:AddLine(newText, 0.3, 0.3, 1.0);	
			end
			if getn(mayLearnRecipe)  > 0 then
				newText = L["Learnable"] .. ": "
				for k2, v2 in pairs(mayLearnRecipe) do
					newText = newText .. " " .. v2
				end
				GameTooltip:AddLine(newText, 0.3, 1.0, 0.3);	
			end			
			GameTooltip:Show()
		end
	end
end

function ProfessionsBook:RemoveDuplicates(knowingRecipe, mayLearnRecipe)
	local newList = {}
	local k1, v1, k2, v2, double
	
	for k1,v1 in pairs(mayLearnRecipe) do
		double = false
		for k2,v2 in pairs(knowingRecipe) do
			if v1 == v2 then
				double = true
			end
		end
		if double == false then
			table.insert(newList, v1)
		end
	end
	return newList
end

function ProfessionsBook:SetAuctionColors()
	local i, itemLink, itemType, itemSubType, itemLevel, buttonName, myTooltip
	local knowingRecipe = {}
	local mayLearnRecipe = {}
	
	auctionCount, _ = GetNumAuctionItems("list")
	for i = 1, PROFESSIONSBOOK_AUCTION_DISPLAY_COUNT do
		buttonName = "BrowseButton"..i
		itemLink = GetAuctionItemLink("list", i + FauxScrollFrame_GetOffset(BrowseScrollFrame))
		if itemLink ~= nil then
			_, _, _, _, _, itemType, itemSubType, _, _, _ = GetItemInfo(itemLink) 
			if itemType == L["Recipe"] then
				myTooltip = getglobal("ProfessionsBookTooltip")
				myTooltip:ClearLines()
				myTooltip:SetAuctionItem("list", i + FauxScrollFrame_GetOffset(BrowseScrollFrame))
				itemLevel = self:GetSkillLevel("ProfessionsBookTooltip", itemSubType)
				knowingRecipe = self:GetCharsKnowingRecipe(self:ConvertRecipeToItem(itemLink), itemSubType)
				mayLearnRecipe = self:GetCharsMayLearnRecipe(itemSubType, 1)
				if getn(mayLearnRecipe) > 0 then
					iconTexture = getglobal(buttonName.."ItemIconTexture")
					iconTexture:SetVertexColor(1.0, 1.0, 0.1)
				end				
				mayLearnRecipe = self:GetCharsMayLearnRecipe(itemSubType, itemLevel)
				if getn(mayLearnRecipe) > 0 then
					iconTexture = getglobal(buttonName.."ItemIconTexture")
					iconTexture:SetVertexColor(0.3, 1.0, 0.3)
				end
				if getn(knowingRecipe) > 0 then
					iconTexture = getglobal(buttonName.."ItemIconTexture")
					iconTexture:SetVertexColor(0.3, 0.3, 1.0)
				end				
			end
		end
	end
end

function ProfessionsBook:GetSkillLevel(tooltip, skillName)
	local i, line, text, a, b, skill, pattern, myTooltip

	myTooltip = getglobal(tooltip)
	for i=1,myTooltip:NumLines() do
		line = getglobal(tooltip .. "TextLeft" .. i)
		text = line:GetText()
		pattern = "%w+ " .. skillName .. " %((%w+)%)"
		a, b, skill = string.find(text, pattern)
		if skill ~= nil then
			return tonumber(skill)
		end
	end
	return -1
end

function ProfessionsBook:IsRecipe(tooltip)
	local i, line, text, a, b, skill, pattern, myTooltip

	myTooltip = getglobal(tooltip)
	for i=1,myTooltip:NumLines() do
		line = getglobal(tooltip .. "TextLeft" .. i)
		text = line:GetText()
		pattern = "%w+ " .. skillName .. " %((%w+)%)"
		a, b, skill = string.find(text, pattern)
		if skill ~= nil then
			return tonumber(skill)
		end
	end
	return -1
end

function ProfessionsBook:GetCharsKnowingRecipe(name, skillName)
	local k1, v1
	local charList = {}
	
	for k1,v1 in pairs(self.db.realm.chars) do
		for k2, v2 in pairs(self.db.realm.chars[k1].tradeSkills[skillName].tradeItems) do
			if v2.tradeItemName == name then
				table.insert(charList, k1)
			end
		end
	end
	table.sort(charList)
	return charList
end

function ProfessionsBook:GetCharsMayLearnRecipe(skillName, skillLevel)
	local charList = {}
	local k, v
	
	for k,v in pairs(self.db.realm.chars) do
		if skillLevel <= self.db.realm.chars[k].tradeSkills[skillName].level then
			table.insert(charList, k)
		end
	end
	table.sort(charList)
	return charList
end

function ProfessionsBook:ConvertRecipeToItem(itemLink)
	local a, b
	
	a, _ = string.find(itemLink, ': ') 
	b, _ = string.find(itemLink, ']')
	if (a ~= nil) and (b ~= nil) then
		return string.sub(itemLink, a + 2, b - 1)
	else
		return ""
	end
end

function ProfessionsBook:GetStoredTradeItemCount(playerName, tradeSkillName)
	local k,v, count
	
	count = 0
	for k,v in pairs(self.db.realm.chars[playerName].tradeSkills[tradeSkillName].tradeItems) do
		count = count + 1
	end
	return count
end

function ProfessionsBook:Search(arg)
	local k1,v1,k2,v2,k3,v3, s, e
	
	searchResult = {}
	for k1,v2 in pairs(self.db.realm.chars) do
		for k2,v2 in pairs(self.db.realm.chars[k1].tradeSkills) do
			for k3,v3 in pairs(self.db.realm.chars[k1].tradeSkills[k2].tradeItems) do
				s, e = string.find(string.lower(v3.tradeItemName), string.lower(arg))
				if  (s ~= nil) and (v3.tradeItemType ~= "header") and (s ~="") then
					searchElement = {}
					searchElement.charName = k1
					searchElement.tradeSkill = k2
					searchElement.id = k3
					searchElement.tradeItemName = v3.tradeItemName
					table.insert(searchResult, searchElement)
				end
			end
		end
	end
	return table.getn(searchResult)
end

function ProfessionsBook:TradeItemExists(playerName, tradeSkillName, tradeItemName)
	local k, v
	
	for k,v in pairs(self.db.realm.chars[playerName].tradeSkills[tradeSkillName].tradeItems) do
		if v.tradeItemName == tradeItemName then
			return true
		end
	end
	return false
end

function ProfessionsBook:VerifiyString(string)
	if (string ~= "") and (string ~= nil) then 
		return true
	end
	return false
end

function ProfessionsBook:VerifiyNumber(number)
	if (number ~= nil) and (number >= 0) then 
		return true
	end
	return false
end

function ProfessionsBook:ScanSkills(type)
	local playerName, tradeSkillName, currentLevel, tools, error
	local tradeItemName, tradeItemType, tradeItemLink
	local reagentName, reagentCount, reagentItemLink, totalReagentCount, tradeItemRecipeLink
	local i, j, tradeItemCount, storedTradeItemCount, oldTradeItemCount, headerCount, newTradeSkillsCount
	local reagent = {}
	local reagents = {}
	
	newTradeSkillsCount = 0
	if type == "Enchanting" then
		tradeItemCount = GetNumCrafts()
		tradeSkillName, currentLevel, _ = GetCraftDisplaySkillLine()
	else
		tradeItemCount = GetNumTradeSkills()
		tradeSkillName, currentLevel, _ = GetTradeSkillLine()
	end
	playerName = UnitName("player")	
	self.db.realm.chars[playerName].tradeSkills[tradeSkillName].level = currentLevel
	oldTradeItemCount = self.db.realm.chars[playerName].tradeSkills[tradeSkillName].totalCount
	headerCount = 0
	error = false
	if tradeSkillName == "UNKNOWN" then
		self:DebugPrint("Trade skill error")
		rescanCount = rescanCount + 1
		error = true
	elseif tradeItemCount >= oldTradeItemCount then
		for i = 1, tradeItemCount do
			if type == "Enchanting" then
				tradeItemName, _, tradeItemType, _, _, _, _ = GetCraftInfo(i)
				tradeItemLink = GetCraftItemLink(i)
				tools = GetCraftSpellFocus(i)
				totalReagentCount = GetCraftNumReagents(i)
			else
				tradeItemName, tradeItemType, _, _ = GetTradeSkillInfo(i)
				if tradeItemType == "header" then
					tradeItemLink = "none"
					totalReagentCount = 0
				else
					tradeItemLink = GetTradeSkillItemLink(i)
					tools = GetTradeSkillTools(i)
					totalReagentCount = GetTradeSkillNumReagents(i)
					tradeItemRecipeLink = GetTradeSkillRecipeLink(i)
				end
			end		
			if (self:VerifiyString(tradeItemName) == false) or (self:VerifiyString(tradeItemType) == false) or (self:VerifiyString(tradeItemLink) == false) or (self:VerifiyNumber(totalReagentCount) == false) then
				self:DebugPrint("Trade Item error")	
				rescanCount = rescanCount + 1				
				error = true
			end
			reagents = {}
			for j = 1, totalReagentCount do
				reagent = {}
				if type == "Enchanting" then
					reagentName, _, reagentCount, _ = GetCraftReagentInfo(i, j)
					reagentItemLink = GetCraftReagentItemLink(i, j)
				else
					reagentName, _, reagentCount, _ = GetTradeSkillReagentInfo(i, j)
					reagentItemLink = GetTradeSkillReagentItemLink(i, j)
				end
				if (self:VerifiyString(reagentName) == true) and (self:VerifiyNumber(reagentCount) == true) and (self:VerifiyString(reagentItemLink) == true) then
					reagent["reagentName"] = reagentName
					reagent["reagentCount"] = reagentCount
					reagent["reagentItemLink"] = reagentItemLink
					table.insert(reagents, j, reagent)
				else
					self:DebugPrint("Reagent error")
					rescanCount = rescanCount + 1
					error = true
				end
			end
			if error == true then
				break
			else	
				if (tradeItemType == "header") then
					headerCount = headerCount + 1
					self.db.realm.chars[playerName].tradeSkills[tradeSkillName].tradeItems["id" .. i].tradeItemName = tradeItemName
					self.db.realm.chars[playerName].tradeSkills[tradeSkillName].tradeItems["id" .. i].tradeItemType = tradeItemType
				else
					if self:TradeItemExists(playerName, tradeSkillName, tradeItemName) == false then				
						newTradeSkillsCount = newTradeSkillsCount + 1
					end
					self.db.realm.chars[playerName].tradeSkills[tradeSkillName].tradeItems["id" .. i].tradeItemName= tradeItemName
					self.db.realm.chars[playerName].tradeSkills[tradeSkillName].tradeItems["id" .. i].tradeItemType = tradeItemType
					self.db.realm.chars[playerName].tradeSkills[tradeSkillName].tradeItems["id" .. i].tradeItemLink = tradeItemLink
					if (type ~= "Enchanting") then
						self.db.realm.chars[playerName].tradeSkills[tradeSkillName].tradeItems["id" .. i].tradeItemRecipeLink = tradeItemRecipeLink
					end
					if ((type == "Enchanting") and (tools ~= 1)) or (type ~="Enchanting") then
						self.db.realm.chars[playerName].tradeSkills[tradeSkillName].tradeItems["id" .. i].tools = tools
					end
					for j=1, totalReagentCount do
						reagent = reagents[j]
						self.db.realm.chars[playerName].tradeSkills[tradeSkillName].tradeItems["id" .. i].reagents["id" .. j].reagentItemName = reagent["reagentName"]
						self.db.realm.chars[playerName].tradeSkills[tradeSkillName].tradeItems["id" .. i].reagents["id" .. j].count = reagent["reagentCount"]
						self.db.realm.chars[playerName].tradeSkills[tradeSkillName].tradeItems["id" .. i].reagents["id" .. j].reagentItemLink = reagent["reagentItemLink"]
					end
					for j=totalReagentCount + 1, 8 do
						self.db.realm.chars[playerName].tradeSkills[tradeSkillName].tradeItems["id" .. i].reagents["id" .. j].reagentItemName = ""
						self.db.realm.chars[playerName].tradeSkills[tradeSkillName].tradeItems["id" .. i].reagents["id" .. j].count = 0
						self.db.realm.chars[playerName].tradeSkills[tradeSkillName].tradeItems["id" .. i].reagents["id" .. j].reagentItemLink = ""
					end
				end
			end
		end
	end
	if error == true then
		if rescanCount == 1 then
			self:DebugPrint(L["Error. Rescanning."])
		end
		rescan = true
	elseif rescan == true then
		self:DebugPrint(L["Rescan successfull."])
		rescanCount = 0
		rescan = false
	end
	if (newTradeSkillsCount > 0) and (rescan == false) then
		self:Print(newTradeSkillsCount .. " " .. L["new recipes found."])
		self.db.realm.chars[playerName].tradeSkills[tradeSkillName].totalCount = tradeItemCount
		rescanCount = 0
	end
end

-- Add header name to header expanded table. 
function ProfessionsBook:SetExpanded(header, expanded)
	local k, v
	local found = false
	
	for k, v in pairs(expandedHeaders) do
		if v == header then
			found = true
		end
	end
	if found == false then
		table.insert(expandedHeaders, header)
	end
end

function ProfessionsBook:GetExpanded(header)
	local k, v
	local found = false

	for k, v in pairs(expandedHeaders) do
		if v == header then
			return true
		end
	end
	return false
end

-- Fill header table
function ProfessionsBook:GetTradeItemHeaders()
	local i, k, v

	i = 1
	for k, v in pairs(ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems) do
		if v.tradeItemType == "header" then
			ProfessionsBook:SetExpanded(v.tradeItemName, true)
		end
	end
end

function ProfessionsBook:GetTradeItemCount()
	local i, count
	
	i = 1
	count = 0
	while true do
		id = "id" .. i
		tradeItemName = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].tradeItemName
		tradeItemType = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].tradeItemType
		-- Break if no more tradeskills
		if tradeItemName == "" then
			break
		end
		if tradeItemType == "header" then
			expanded = ProfessionsBook:GetExpanded(tradeItemName)
			count = count + 1
		end
		if (tradeItemType ~= "header") and (expanded == true) then
			count = count + 1
		end
		i = i + 1
	end
	return count
end

function ProfessionsBook:GetFilteredTradeItemList(tradeItemFilter)
	local i, count, addTradeItems
	
	i = 1
	count = 0
	while true do
		id = "id" .. i
		tradeItemName = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].tradeItemName
		tradeItemType = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].tradeItemType
		-- Break if no more tradeskills
		if tradeItemName == "" then
			break
		end
		if (tradeItemName == tradeItemFilter) then
			table.insert(tradeItemList, id)
			addTradeItems = true
			count = count + 1
		end
		if (tradeItemType == "header") and (tradeItemName ~= tradeItemFilter) then
			addTradeItems = false
		end
		-- Add trade item to list
		if (tradeItemType ~= "header") and (addTradeItems == true) then
			if ProfessionsBook:InPlaceFilter(id) == true then
				table.insert(tradeItemList , id) 
				count = count + 1
			end
		end
		i = i + 1
	end
	-- Remove headers with no trade items contained
	tradeItemCount = ProfessionsBook:RemoveEmptyHeaders(count)
end

function ProfessionsBook:GetItemPlace(id)
		local itemLink, itemName, itemEquipLoc
		local a, b

		itemLink = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].tradeItemLink
		itemName = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].tradeItemName
		-- WoW has no function to get the equip place a enchanting can target. So we have to parse the string of the enchanting itself to geht the place. Seperator is "-". (Format: "Target - Enchanting")
		if L["Enchanting"] == ProfessionsBook.selectedTradeSkill then
			a, b = string.find(itemName, "-")
			-- Entchantment is something to be produced and has no enchanting target.
			if a == nil then
				return L[""]
			else
				return string.sub(itemName, 1, a - 2)
			end
		else
			_, _, _, _, _, _, _, _, itemEquipLoc, _ = GetItemInfo(itemLink)
		end

		return L[itemEquipLoc]
end

function ProfessionsBook:GetTradeItemList()
	local i, count
	
	i = 1
	count = 0
	while true do
		id = "id" .. i
		tradeItemName = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].tradeItemName
		tradeItemType = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].tradeItemType
		-- Break if no more tradeskills
		if tradeItemName == "" then
			break
		end
		-- Add header to list
		if tradeItemType == "header" then
			expanded = ProfessionsBook:GetExpanded(tradeItemName)
			table.insert(tradeItemList , id) 
			count = count + 1
		end
		-- Add trade item to list if its header is expanded
		if ((tradeItemType ~= "header") and (expanded == true)) or (L["Enchanting"] == ProfessionsBook.selectedTradeSkill) then
			if ProfessionsBook:InPlaceFilter(id) == true then
				table.insert(tradeItemList , id) 
				count = count + 1
			end
		end
		i = i + 1
	end
	-- Remove headers with no trade items contained
	tradeItemCount = ProfessionsBook:RemoveEmptyHeaders(count)
end

function ProfessionsBook:FillPlacesList()
	local k, v, k2, v2, tradeItemType, place, found

	placesList = {}
	-- Get places of all displayed trade items
	for k, v in pairs(tradeItemList) do
		tradeItemType = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[v].tradeItemType
		if tradeItemType ~= "header" then
			place = ProfessionsBook:GetItemPlace(v)
			found = false
			-- Is place allready in the list?
			for k2, v2 in pairs(placesList) do
				if place == v2 then
					found = true
				end
			end
			-- If place is not in list, add it.
		if found == false then
			table.insert(placesList, place)
			end
		end
	end
end

function ProfessionsBook:FillDetails(id)
	local tradeItemName
	
	tradeItemName = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].tradeItemName
	LabelTradeItemName:SetText(tradeItemName)
end


function ProfessionsBook:InPlaceFilter(id)
	local filter
	
	filter = UIDropDownMenu_GetText(ProfessionsBookPlacesDropDownMenu)
	if filter == L["All places"] then
		return true
	end

	if ProfessionsBook:GetItemPlace(id) == filter then
		return true
	end
	return false
end

function ProfessionsBook:CalculateMinimapCoords(degr)
	local x, y
	
	y = sin(degr ) * 80
	x = y / tan(degr)
	return x + 10, y + 24
end

function ProfessionsBook:RemoveEmptyHeaders(size)
	local tradeItemListTemp = {}
	local i, tradeItemType1, tradeItemType2, tradeItemName, count
	
	if size == 0 then
		return 0
	end
	count = 0
	-- Check all entries but the last one of tradeItemList if two headers are in sequence with no trade items between.
	for i = 1, size - 1 do
		tradeItemType1 = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[tradeItemList[i]].tradeItemType
		tradeItemType2 = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[tradeItemList[i + 1]].tradeItemType		
		tradeItemName = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[tradeItemList[i]].tradeItemName
		if ((tradeItemType1 == "header") and (tradeItemType2 ~= "header")) or (tradeItemType1 ~= "header")  or ((tradeItemType1 == "header") and (tradeItemType2 == "header") and (ProfessionsBook:GetExpanded(tradeItemName) == false))then
			table.insert(tradeItemListTemp, tradeItemList[i])
			count = count + 1
		end
	end
	-- The algorithm above checks all entries of tradeItemList for headers in sequence. However the last entry of that list is not checked. 
	tradeItemType1 = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[tradeItemList[size]].tradeItemType
	tradeItemName = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[tradeItemList[size]].tradeItemName
	if (tradeItemType1 ~= "header") or (ProfessionsBook:GetExpanded(tradeItemName) == false) then		
		table.insert(tradeItemListTemp, tradeItemList[size])
		count = count + 1
	end	
	tradeItemList = tradeItemListTemp
	return count
end

-- Update trade skill selection buttons corresponding to the selected character
function ProfessionsBook:UpdateTradeSkillSelection()
	local i
	local button
	local k, v
	
	for i = 1, PROFESSIONSBOOK_TRADESKILL_BUTTON_COUNT do
		button = getglobal("ProfessionsBookTradeSkillButton" .. i)
		button:Hide()
	end
	i = 1
	for k,v in pairs(ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills) do 
		if k ~= "UNKNOWN" then
			button = getglobal("ProfessionsBookTradeSkillButton" .. i)
			button:SetText(k)
			button:Show()
			i = i + 1
		end
	end
end

function ProfessionsBook:UpdateTradeItemButtons()
	local i, k, v, id
	local expanded = true
	local tradeItemName, tradeItemType
	local button, color, buttonCount

	-- Clear alle trade item buttons
	for i = 1, PROFESSIONSBOOK_TRADEITEM_BUTTON_COUNT do
		button = getglobal("ProfessionsBookTradeItemButton" .. i)
		button:SetText("")
		button:Hide()		
	end
	
	TradeItemHighlightFrame:Hide()	
	buttonCount = 1
	for k,v in pairs(tradeItemList) do 
		tradeItemName = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[v].tradeItemName
		tradeItemType = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[v].tradeItemType		
		button = getglobal("ProfessionsBookTradeItemButton" .. buttonCount)
		if (k > FauxScrollFrame_GetOffset(TradeItemListScrollFrame)) and (k <= FauxScrollFrame_GetOffset(TradeItemListScrollFrame) + PROFESSIONSBOOK_TRADEITEM_BUTTON_COUNT) then
			if tradeItemType == "header" then
				expanded = ProfessionsBook:GetExpanded(tradeItemName)
				if expanded == true then
					button:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up")
				else
					button:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up")
				end	
			else
				button:SetNormalTexture("")
				button:SetHighlightTexture("")	
				-- Place the highlight and lock the highlight state
				if ( selectedTradeItem == v ) then
					TradeItemHighlightFrame:SetPoint("TOPLEFT", "ProfessionsBookTradeItemButton"..buttonCount, "TOPLEFT", 0, 0);
					TradeItemHighlightFrame:Show()
					button:LockHighlight()
				else
					button:UnlockHighlight()
				end						
			end
			button:SetText(tradeItemName)
			button:Show()
			id = string.sub(v, 3)
			button:SetID(id)
			color = TradeSkillTypeColor[tradeItemType];
			if ( color ) then
				button:SetTextColor(color.r, color.g, color.b);
			end
			if ( TradeItemListScrollFrame:IsShown() ) then
				button:SetWidth(283);
			else
				button:SetWidth(323);
			end	
			buttonCount = buttonCount + 1	
		end
	end
	if tradeItemCount > PROFESSIONSBOOK_TRADEITEM_BUTTON_COUNT then
		TradeItemListScrollFrame:Show()
	else
		TradeItemListScrollFrame:Hide()
	end
end

function ProfessionsBook:GetCharacterCount()
	local k,v, count
	
	count = 0
	for k,v in pairs(ProfessionsBook.db.realm.chars) do
		count = count + 1
	end
	return count
end

function ProfessionsBook:UpdateSearchFrame(resultsCount)
	local buttonCount , id, button, i
	
	buttonCount = 1
	for i = 1,PROFESSIONSBOOK_SEARCHITEM_BUTTON_COUNT do
		button = getglobal("ProfessionsBookSearchResultButton" .. i)
		button:Hide()
	end
	SearchResultHighlightFrame:Hide()
	for k,v in pairs(searchResult) do
		button = getglobal("ProfessionsBookSearchResultButton" .. buttonCount)
		if (k > FauxScrollFrame_GetOffset(SearchResultScrollFrame)) and (k <= FauxScrollFrame_GetOffset(SearchResultScrollFrame) + PROFESSIONSBOOK_SEARCHITEM_BUTTON_COUNT) then		
			button:SetNormalTexture("")
			button:SetHighlightTexture("")	
			-- Place the highlight and lock the highlight state
			if ( selectedSearchItem == v.charName .. ": " .. v.tradeItemName ) then
				SearchResultHighlightFrame:SetPoint("TOPLEFT", "ProfessionsBookSearchResultButton"..buttonCount, "TOPLEFT", 15, 0);
				SearchResultHighlightFrame:Show()
				button:LockHighlight()
			else
				button:UnlockHighlight()						
			end
			button:SetText(v.charName .. ": " .. v.tradeItemName)
			button:Show()
			id = k
			button:SetID(id)
			color = TradeSkillTypeColor["medium"]
			button:SetTextColor(color.r, color.g, color.b)
			if ( SearchResultScrollFrame:IsShown() ) then
				button:SetWidth(303);
			else
				button:SetWidth(323);
			end	
			buttonCount = buttonCount + 1			
		end
	end
	if resultsCount > PROFESSIONSBOOK_SEARCHITEM_BUTTON_COUNT then
		SearchResultScrollFrame:Show()
	else
		SearchResultScrollFrame:Hide()
	end
end

function ProfessionsBookFrame_Update()
	FauxScrollFrame_Update(TradeItemListScrollFrame, tradeItemCount, PROFESSIONSBOOK_TRADEITEM_BUTTON_COUNT, PROFESSIONSBOOK_TRADEITEM_HEIGHT);
	ProfessionsBook:UpdateTradeItemButtons()
end

function ProfessionsBookSearch_Update()
	FauxScrollFrame_Update(SearchResultScrollFrame, table.getn(searchResult), PROFESSIONSBOOK_SEARCHITEM_BUTTON_COUNT, PROFESSIONSBOOK_SEARCHITEM_HEIGHT);
	ProfessionsBook:UpdateSearchFrame(table.getn(searchResult))
end

-- Create characters selection dropdown menu
function ProfessionsBook_InitMinimapSlider(slider)
	local sl = getglobal(slider)

	getglobal(slider .. "Low"):SetText("1")
	getglobal(slider .. "High"):SetText("360")
	getglobal(slider .. "Text"):SetText(L["Minimap position"])
end

function ProfessionsBookCharacterDropDownMenu_Initialize()
        local i = 0
		local k, v
		local info = UIDropDownMenu_CreateInfo() 
		local charNames = {}
		
		for k,v in pairs(ProfessionsBook.db.realm.chars) do 
			table.insert(charNames, k)
		end
		table.sort(charNames)
		for k,v in pairs(charNames) do 
			info.text = v
			info.value = i
			info.func = ProfessionsBookCharacterDropDownMenu_OnClick
			info.owner = this:GetParent()
			info.checked = nil
			info.icon = nil
			UIDropDownMenu_AddButton(info, 1)
			i = i + 1
		end
		TradeItemListScrollFrame:Hide()
		TradeItemExpandButtonFrame:Hide()
end

function ProfessionsBookChannelsDropDownMenu_Initialize()
        local i = 0
		local k, v
		local info = UIDropDownMenu_CreateInfo() 

		for k,v in pairs(channels) do 
			info.text = L[v]
			info.value = i
			info.func = ProfessionsBookChannelsDropDownMenu_OnClick
			info.owner = this:GetParent()
			info.checked = nil
			info.icon = nil
			UIDropDownMenu_AddButton(info, 1)
			i = i + 1
		end
end

function ProfessionsBookTradeTypeDropDownMenu_Initialize()
        local i = 1
		local k, v
		local info = UIDropDownMenu_CreateInfo() 

		info.text = L["All subclasses"]
		info.value = 0
		info.func = ProfessionsBookTradeTypeDropDownMenu_OnClick
		info.owner = this:GetParent()
		info.checked = nil
		info.icon = nil
		UIDropDownMenu_AddButton(info, 1)		
		for k,v in pairs(allHeaders) do 
			info.text = v
			info.value = i
			info.func = ProfessionsBookTradeTypeDropDownMenu_OnClick
			info.owner = this:GetParent()
			info.checked = nil
			info.icon = nil
			UIDropDownMenu_AddButton(info, 1)
			i = i + 1
		end
end

function ProfessionsBookPlacesDropDownMenu_Initialize()
	local i = 1
	local k, v
	local info = UIDropDownMenu_CreateInfo() 

	info.text = L["All places"]
	info.value = 0
	info.func = ProfessionsBookPlacesDropDownMenu_OnClick
	info.owner = this:GetParent()
	info.checked = nil
	info.icon = nil
	UIDropDownMenu_AddButton(info, 1)	
	ProfessionsBook:FillPlacesList()
	for k,v in pairs(placesList) do 
		info.text = v
		info.value = i
		info.func = ProfessionsBookPlacesDropDownMenu_OnClick
		info.owner = this:GetParent()
		info.checked = nil
		info.icon = nil
		UIDropDownMenu_AddButton(info, 1)
		i = i + 1
	end	
end

function ProfessionsBookCharacterDropDownMenu_OnClick()
	local i, button

	UIDropDownMenu_SetSelectedValue(this.owner, this.value)
	ProfessionsBook.db.realm.selectedChar = UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)
	ProfessionsBook:UpdateTradeSkillSelection()
	
	for i = 1, PROFESSIONSBOOK_TRADEITEM_BUTTON_COUNT do
		button = getglobal("ProfessionsBookTradeItemButton" .. i)
		button:Hide()
	end
	TradeItemHighlightFrame:Hide()
	for i = 1, PROFESSIONSBOOK_TRADESKILL_BUTTON_COUNT do
		button = getglobal("ProfessionsBookTradeSkillButton" .. i)
		button:UnlockHighlight()
	end
	ProfessionsBookDetailScrollFrame:Hide()
	ProfessionsBookSendButton:Disable()
	SkillLevel:Hide()
end 

function ProfessionsBookChannelsDropDownMenu_OnClick()
	UIDropDownMenu_SetSelectedValue(this.owner, this.value)
	ProfessionsBook:UpdateTradeSkillSelection()
	selectedChannel = this.value
end 

function ProfessionsBookTradeTypeDropDownMenu_OnClick()
	UIDropDownMenu_SetSelectedValue(this.owner, this.value)	
	expandedHeaders = {}
	tradeItemList = {}
	-- "All subclasses" selected
	if this.value == 0 then
		ProfessionsBook:GetTradeItemHeaders()
		ProfessionsBook:GetTradeItemList()	
	else
		table.insert(expandedHeaders, UIDropDownMenu_GetText(ProfessionsBookTradeTypeDropDownMenu))
		ProfessionsBook:GetFilteredTradeItemList(UIDropDownMenu_GetText(ProfessionsBookTradeTypeDropDownMenu))
	end
	ProfessionsBookFrame_Update()
	ProfessionsBook:UpdateTradeItemButtons()
end 

function ProfessionsBookPlacesDropDownMenu_OnClick()
	UIDropDownMenu_SetSelectedValue(this.owner, this.value)	
		if ProfessionsBook.selectedTradSkill ~= L["Enchanting"] then
			expandedHeaders = {}
			tradeItemList = {}
			-- "All subclasses" selected
			if UIDropDownMenu_GetText(ProfessionsBookTradeTypeDropDownMenu) == L["All subclasses"] then
				ProfessionsBook:GetTradeItemHeaders()
				ProfessionsBook:GetTradeItemList()	
			else
				table.insert(expandedHeaders, UIDropDownMenu_GetText(ProfessionsBookTradeTypeDropDownMenu))
				ProfessionsBook:GetFilteredTradeItemList(UIDropDownMenu_GetText(ProfessionsBookTradeTypeDropDownMenu))
			end
		ProfessionsBookFrame_Update()
		ProfessionsBook:UpdateTradeItemButtons()
	end
end 

function ProfessionsBookTradeSkillButton_OnClick(id, offset)
	local button, i

	for i = 1, PROFESSIONSBOOK_TRADESKILL_BUTTON_COUNT do
		button= getglobal("ProfessionsBookTradeSkillButton" .. i)
		button:UnlockHighlight()
	end
	expandedHeaders = {}
	button = getglobal("ProfessionsBookTradeSkillButton" .. id)
	button:LockHighlight()
	TradeItemExpandButtonFrame:Show()
	selectedTradeItem = ""
	ProfessionsBook.selectedTradeSkill = button:GetText()
	ProfessionsBook:GetTradeItemHeaders()	
	FauxScrollFrame_SetOffset(TradeItemListScrollFrame, offset);
	tradeItemList = {}
	UIDropDownMenu_Initialize(ProfessionsBookTradeTypeDropDownMenu, ProfessionsBookTradeTypeDropDownMenu_Initialize)
	UIDropDownMenu_SetSelectedValue(ProfessionsBookTradeTypeDropDownMenu, 0)	
	UIDropDownMenu_Initialize(ProfessionsBookPlacesDropDownMenu, ProfessionsBookPlacesDropDownMenu_Initialize)
	UIDropDownMenu_SetSelectedValue(ProfessionsBookPlacesDropDownMenu, 0)	
	ProfessionsBook:GetTradeItemList()
	ProfessionsBookFrame_Update()
	ProfessionsBook:UpdateTradeItemButtons()
	allHeaders = expandedHeaders
	ProfessionsBookDetailScrollFrame:Hide()
	ProfessionsBookSendButton:Disable()
	SkillLevel:SetText(SKILL_LEVEL .. ": " .. ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].level)
	SkillLevel:Show()
end

function ProfessionsBookTradeItemButton_OnClick(arg1)
	local color, tradeItemName, tradeItemType, id, id2, k, v, count, itemTexture
	local reagentTexture, nameLabel, countLabel, reagentName, reagentCount, i, tools

	id = "id" .. arg1
	tradeItemType = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].tradeItemType	
	tradeItemName = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].tradeItemName
	tradeItemLink = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].tradeItemLink
	color = TradeSkillTypeColor[tradeItemType]
	if ( color ) then
		TradeItemHighlight:SetVertexColor(color.r, color.g, color.b)
	end
	selectedTradeItem = "id" .. arg1
	
	-- If header is set to expanded, add header name to expandedHeaders table. Remove from that table otherwise.
	if tradeItemType == "header" then
		ProfessionsBookSendButton:Disable()
		if ProfessionsBook:GetExpanded(tradeItemName) == true then
			for k, v in pairs(expandedHeaders) do
				if v == tradeItemName then
					table.remove(expandedHeaders, k)
				end
			end
		else
			table.insert(expandedHeaders, tradeItemName)
		end
	else
		ProfessionsBookSendButton:Enable()
	end
	tradeItemList = {}
	if UIDropDownMenu_GetText(ProfessionsBookTradeTypeDropDownMenu) == L["All subclasses"] then
		ProfessionsBook:GetTradeItemList()
	else
		ProfessionsBook:GetFilteredTradeItemList(UIDropDownMenu_GetText(ProfessionsBookTradeTypeDropDownMenu))
	end
	ProfessionsBook:UpdateTradeItemButtons()
	
	
	if tradeItemType ~= "header" then
		ProfessionsBookDetailScrollFrame:Show()
		ProfessionsBook:FillDetails(id)
	else
		ProfessionsBookDetailScrollFrame:Hide()
	end
	
	-- Fill details frame
	_, _, _, _, _, _, _, _, _, itemTexture = GetItemInfo(tradeItemLink)
	if ProfessionsBook.selectedTradeSkill == L["Enchanting"] then
		ProfessionsBookTradeItemIcon:SetNormalTexture("Interface\\Icons\\INV_Scroll_04")
	else
		ProfessionsBookTradeItemIcon:SetNormalTexture(itemTexture)
	end
	tools = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].tools
	if (tools == "") or (tools == nil) then
		LabelRequirement:Hide()
		LabelRequirementText:Hide()
	else
		LabelRequirement:Show()
		LabelRequirementText:Show()	
		LabelRequirementText:SetText(tools)
	end
		
	count = 1	
	for k, v in pairs(ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].reagents) do
		id2 = "id" .. count
		reagentCount = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].reagents[id2].count
		if reagentCount ~= 0  then
			reagentLink = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].reagents[id2].reagentItemLink
			reagentName = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].reagents[id2].reagentItemName
			reagentCount = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].reagents[id2].count
			_, _, _, _, _, _, _, _, _, reagentTexture = GetItemInfo(reagentLink)
			button = getglobal("ProfessionsBookTradeItemReagent" .. count)
			nameLabel = getglobal("ProfessionsBookTradeItemReagent"..count.."Name")
			countLabel = getglobal("ProfessionsBookTradeItemReagent"..count.."Count");
			SetItemButtonTexture(button, reagentTexture)
			nameLabel:SetText(reagentName)
			countLabel:SetText(reagentCount)
			button:Show()
			count = count + 1
		end
	end
	for i = count, 8 do
		button = getglobal("ProfessionsBookTradeItemReagent" .. i)
		button:Hide()
	end
	ProfessionsBookDetailScrollFrame:UpdateScrollChildRect()
end

function ProfessionsBookTradeItemButton_OnShiftClick(arg1)
	local id
	
	id = "id" .. arg1
	if ProfessionsBook.selectedTradeSkill == L["Enchanting"] then
		ChatEdit_InsertLink(ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].tradeItemLink)
	else
		ChatEdit_InsertLink(ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[id].tradeItemRecipeLink)
	end
end

function ProfessionsBookOpenSearchButton_OnClick()
	if searchFrameIsShown == true then
		ProfessionsBook:HideSearchFrame()
	else
		ProfessionsBook:ShowSearchFrame()
	end
end

function ProfessionsBookSearchButton_OnClick()
	local resultsCount
	
	resultsCount = ProfessionsBook:Search(ProfessionsBookSearchBox:GetText())
	FauxScrollFrame_Update(SearchResultScrollFrame, resultsCount, PROFESSIONSBOOK_SEARCHITEM_BUTTON_COUNT, PROFESSIONSBOOK_SEARCHITEM_HEIGHT);
	FauxScrollFrame_SetOffset(SearchResultScrollFrame, 0)
	if (resultsCount <= PROFESSIONSBOOK_SEARCHITEM_BUTTON_COUNT) then
		SearchResultScrollFrame:Hide()
	else
		SearchResultScrollFrame:Show()
	end
	selectedSearchItem = "none"
	ProfessionsBook:UpdateSearchFrame(resultsCount)
end

function ProfessionsBookSearchResultButton_OnClick(id)
	local i, charName, tradeSkill, itemID, button, offset
	
	charName = searchResult[id].charName
	tradeSkill = searchResult[id].tradeSkill
	itemID = searchResult[id].id
	offset = tonumber(string.sub(itemID, 3))
	selectedSearchItem = this:GetText()
	ProfessionsBook:UpdateSearchFrame(table.getn(searchResult))
	UIDropDownMenu_Initialize(ProfessionsBookCharacterDropDownMenu, ProfessionsBookCharacterDropDownMenu_Initialize)
	for i = 0, ProfessionsBook:GetCharacterCount() do
		UIDropDownMenu_SetSelectedValue(ProfessionsBookCharacterDropDownMenu, i)
		if UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu) == charName then
			break
		end
	end
	ProfessionsBook:UpdateTradeSkillSelection()
	for i = 1,PROFESSIONSBOOK_TRADESKILL_BUTTON_COUNT do 
		button = getglobal("ProfessionsBookTradeSkillButton" .. i)
		if button:GetText() == tradeSkill then
			ProfessionsBookTradeSkillButton_OnClick(i, 0)
		end
	end
	selectedTradeItem = id
	--FauxScrollFrame_SetOffset(TradeItemListScrollFrame, offset)
	--TradeItemListScrollFrame:SetVerticalScroll(offset * PROFESSIONSBOOK_TRADEITEM_HEIGHT)
	ProfessionsBookTradeItemButton_OnClick(offset)
end

function ProfessionsBookTradeItemCollapseAllButton_OnClick()
	if (this.collapsed) then
		this.collapsed = nil;
		TradeItemCollapseAllButton:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up")
		if UIDropDownMenu_GetText(ProfessionsBookTradeTypeDropDownMenu) == L["All subclasses"] then
			ProfessionsBook:GetTradeItemHeaders()	
		else
			table.insert(expandedHeaders, UIDropDownMenu_GetText(ProfessionsBookTradeTypeDropDownMenu))
		end
	else
		this.collapsed = 1;
		TradeItemCollapseAllButton:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up")
		expandedHeaders = {}
	end
	tradeItemList = {}
	if UIDropDownMenu_GetText(ProfessionsBookTradeTypeDropDownMenu) == L["All subclasses"] then
		ProfessionsBook:GetTradeItemList()
	else
		ProfessionsBook:GetFilteredTradeItemList(UIDropDownMenu_GetText(ProfessionsBookTradeTypeDropDownMenu))
	end
	ProfessionsBook:UpdateTradeItemButtons()
end

function ProfessionsBookSendButton_OnClick()
	local tradeItemLink, reagentItemLink, reagentItemCount
	
	tradeItemLink = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[selectedTradeItem].tradeItemLink
	SendChatMessage(L["Materials for"] .. " " .. tradeItemLink .. ":", channels[selectedChannel+1])
	for k, v in pairs(ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[selectedTradeItem].reagents) do
		reagentItemLink = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[selectedTradeItem].reagents[k].reagentItemLink
		reagentItemCount = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[selectedTradeItem].reagents[k].count
		if reagentItemCount > 0 then
			SendChatMessage("  " .. reagentItemLink .. " x " .. reagentItemCount, channels[selectedChannel+1])
		end
	end
end

function ProfessionsBookOptionsButton_OnClick()
	ProfessionsBookOptionsDialog:Show()
end

function ProfessionsBookMinimapButton_OnClick()
	ProfessionsBook:ShowFrame()
end

function ProfessionsBookShowMinimapCheckButton__OnClick()
	local playerName

	playerName= UnitName("player")
	if ProfessionsBookShowMinimapCheckButton:GetChecked() == 1 then
		MinimapButton:Show()
		ProfessionsBook.db.realm.chars[playerName].minimap = 1
	else
		MinimapButton:Hide()
		ProfessionsBook.db.realm.chars[playerName].minimap = 0
	end
end

function ProfessionsBook_ChangeMinimapSlider()
	local degr, x, y, playerName
	
	playerName= UnitName("player")
	degr = ProfessionsBookMinimapSlider:GetValue()
	x, y = ProfessionsBook:CalculateMinimapCoords(degr)
	MinimapButton:SetPoint("CENTER", "MinimapBackdrop", "CENTER", x, y)
	if ProfessionsBook.db.realm ~= nil then
		ProfessionsBook.db.realm.chars[playerName].degr = degr
	end
end

function ProfessionsBookTradeItemReagentButton_OnShiftClick(arg1)
	local id
	
	id = "id" .. arg1
	ChatEdit_InsertLink(ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[selectedTradeItem].reagents[id].reagentItemLink)
end

function ProfessionsBookTradeItemDetailButton_OnShiftClick()
	local id
	
	id = "id" .. arg1
	ChatEdit_InsertLink(ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[selectedTradeItem].tradeItemLink)
end

function ProfessionsBookTradeItemDetailButton_OnCtrlClick()
	local id
	
	id = "id" .. arg1
	DressUpItemLink(ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[selectedTradeItem].tradeItemLink)
end

function ProfessionsBook_SetTooltip()
	local tradeItemLink
	
	GameTooltip:SetOwner(this, "ANCHOR_TOPLEFT")
	tradeItemLink = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[selectedTradeItem].tradeItemLink
	GameTooltip:SetHyperlink(tradeItemLink)
	CursorUpdate()	
end

function ProfessionsBook_SetReagentTooltip(id)
	local reagentsItemLink

	GameTooltip:SetOwner(this, "ANCHOR_TOPLEFT")
	reagentsItemLink = ProfessionsBook.db.realm.chars[UIDropDownMenu_GetText(ProfessionsBookCharacterDropDownMenu)].tradeSkills[ProfessionsBook.selectedTradeSkill].tradeItems[selectedTradeItem].reagents["id" .. id].reagentItemLink
	GameTooltip:SetHyperlink(reagentsItemLink)
end



